#
# This is a modified EasyBuild easyblock file based on anaconda.py distributed as part of EasyBuild:
# https://github.com/easybuilders/easybuild-easyblocks/blob/5c1748a8213a47c3abc3ff4176d3e9eb2b8547a7/easybuild/easyblocks/a/anaconda.py
# When downloaded, it was distributed under (and thus remain under) the GNU GENERAL PUBLIC LICENSE Version 2:
#   https://github.com/easybuilders/easybuild-easyblocks/blob/5c1748a8213a47c3abc3ff4176d3e9eb2b8547a7/LICENSE
# The following are the original copyright/license headers:
##
# Copyright 2009-2022 Ghent University
#
# This file is part of EasyBuild,
# originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
# with support of Ghent University (http://ugent.be/hpc),
# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
# Flemish Research Foundation (FWO) (http://www.fwo.be/en)
# and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
#
# https://github.com/easybuilders/easybuild
#
# EasyBuild is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation v2.
#
# EasyBuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
##
#
# This file has been modified by the ssetools AUTHORS to install miniforge.
# History:
#   rar@nsc: 2024 initial version
#
"""
EasyBuild support for building and installing Miniforge based on the same installation for Anaconda/Miniconda, implemented as an easyblock
@author: Jillian Rowe (New York University Abu Dhabi)
@author: Kenneth Hoste (HPC-UGent)
@author: Rickard Armiento (NSC)
"""

import os
import stat

from easybuild.easyblocks.generic.binary import Binary
from easybuild.easyblocks.anaconda import EB_Anaconda
from easybuild.tools.filetools import adjust_permissions, remove_dir
from easybuild.tools.run import run_cmd


class EB_Miniforge(EB_Anaconda):

    def sanity_check_step(self):
        """
        Custom sanity check for Miniforge
        """
        custom_paths = {
            'files': [os.path.join('bin', x) for x in ['mamba', 'python']],
            'dirs': ['bin', 'etc', 'lib', 'pkgs'],
        }
        super(EB_Anaconda, self).sanity_check_step(custom_paths=custom_paths)
