#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# install-pproxy.sh
# installs our forked python proxy server from its repository
#
# History:
#   rar@nsc: 2024 initial version

set -e

VERSION="2.7.9"

if [ -z  "${HPC_SSE_SYSNAME}" ]; then
    echo "Environment variable HPC_SSE_SYSNAME must be set."
    exit 1
fi

mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}"
cd "/software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}"

# For upgrades to the latest version, just do pip3 install --user pproxy[accelerated] asyncssh pycryptodome python-daemon to find out what versions of the dependencies that gives
# The dependency versions specified below were tested with pproxy 2.7.9
# PYTHONUSERBASE=./pipenv pip3 install --user pproxy[accelerated]=="${VERSION}" asyncssh==2.14.2 pycryptodome==3.20.0 python-daemon==3.0.1 pycparser==2.21 setuptools==69.1.1 uvloop==0.19.0 asyncssh==2.14.2 cryptography==42.0.5 docutils==0.20.1 typing_extensions==4.10.0 cffi==1.16.0 lockfile==0.12.2

# Edit: actually, install pproxy from forked git repo to get a necessary patch until (if) it is merged upstream
PYTHONUSERBASE=./pipenv pip3 install --user asyncssh==2.14.2 pycryptodome==3.20.0 python-daemon==3.0.1 pycparser==2.21 setuptools==69.1.1 uvloop==0.19.0 asyncssh==2.14.2 cryptography==42.0.5 docutils==0.20.1 typing_extensions==4.10.0 cffi==1.16.0 lockfile==0.12.2
git clone https://gitlab.liu.se/NSC/python-proxy python-proxy-git
cd python-proxy-git
git checkout 26ba54acef57a3f30a392364102e2e9f69c59adb
mkdir bin

cat > bin/pproxy <<EOF
#!/usr/bin/python3
# -*- coding: utf-8 -*-
import re
import sys

from pproxy.server import main
if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw|\.exe)?$', '', sys.argv[0])
    sys.exit(main())
EOF
chmod +x bin/pproxy

cd ..
cat > pproxy <<EOF
#!/bin/bash

PYTHONPATH=/software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}/python-proxy-git PYTHONUSERBASE=/software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}/pipenv exec /software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}/python-proxy-git/bin/pproxy "\$@"
EOF

# The above is to be replaced with this, if pip distributes patched version:
#
#cat > pproxy <<EOF
##!/bin/bash
#
#PYTHONUSERBASE=/software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}/pipenv exec /software/sse2/${HPC_SSE_SYSNAME}/manual/pproxy/${VERSION}/pipenv/bin/pproxy "\$@"
#EOF

chmod +x pproxy

echo "pproxy installed"
