# Bootstrapping the SSE environment

## Optional: mount or sync `/software/sse2` and `/software/sse2/generic`

In the SSE2 filesystem layout, the software in `/software/sse2/generic` is meant to be the same across all HPC systems.
Software that depends on the HPC system is placed under `/software/sse2/generic/<system name>_<os id>`, meaning that a single `/software/sse2` directory can be shared across multiple HPC systems.
Hence, it is possible to mount these directories as shared between systems or set up some form of periodic synchronization.

When bootstrapping a new HPC system, consider whether that should apply to all of `/software/sse2`, just `/software/sse2/generic`, or not at all - and set up the mount or synchronization.

## Install ssetools

This step sets up:

- A directory `/software/sse2/generic/manual/ssetools` with versioned subdirectories, a `dev` subdirectory.
- A directory `/software/sse2/generic/modules/hpc` with, at least, a hidden versioned module file to the most recent version `.<version>.lua` and a `default` symlink to it.

If these directories are already in place because a shared mount or synchronization is used for `/software/sse2` or `/software/sse2/generic` you can skip to the next section.

To setup an SSE v2 environment, we will:

- Clone the ssetools repository into `/software/sse2/generic/manual/ssetools/dev` (having the development version checked out centrally enables testing development versions via loadable modules).
- Check out the latest versioned release.
- Run the provided installation script to create a versioned installation into `/software/sse2/generic/manual/ssetools/<version>`.

Follow these steps:
```
  mkdir -p /software/sse2/generic/manual/ssetools
  git clone git@gitlab.liu.se:NSC/ssetools.git /software/sse2/generic/manual/ssetools/dev
  cd /software/sse2/generic/manual/ssetools/dev
  HPC_SSE_VER=$(git describe --tags --abbrev=0)
  echo "${HPC_SSE_VER}"
```
The last command outputs the latest versioned release, which is in the variable `$HPC_SSE_VER`.
Make sure to keep this variable set in your environment until you have completed the section [SSE initialization scripts](#sse-initialization-scripts) and relogged into the SSE environment (which sets this variable for you).
```
  git checkout "tags/${HPC_SSE_VER}"
  ./install.sh
```
Note that the install script in the last step instructs you to make the installed version of ssetools the default one by executing:
```
ln -sfn ".${HPC_SSE_VER}.lua" "/software/sse2/generic/modules/hpc/default"
```

When the appropriate version of ssetools has been installed, return the repo in the `dev` directory to the head of the main branch:
```
  cd /software/sse2/generic/manual/ssetools/dev
  git checkout main
```

## Define HPC_RESOURCE_NAME

Your initalization needs to somewhere set the environment variable `HPC_RESOURCE_NAME` to a reasonable human-readable "name" of the HPC system with no spaces or other unusual characters.
Either edit the global initalization files to set this variable for all logins, or, set it temporarily in your shell now, and in the later step [SSE initialization scripts](#sse_initialization_scripts
) set it by editing the initalization script installed in that step.
```
export HPC_RESOURCE_NAME="<name>"
```

## Bootstrap Lmod installation

We build and install our own Lua and Lmod to make sure to get tested versions known to work for the features we use.

Prerequisites:

- A system with installed packages for gcc and libreadline-dev
- Installation packages (with sha256 hashes):

  * [lua-5.3.4.tar.gz](https://www.lua.org/ftp/lua-5.3.4.tar.gz) `[f681aa518233bc407e23acf0f5887c884f17436f000d453b2491a9f11a52400c]`
  * [lua-compat-5.2-0.3.tar.gz](https://github.com/keplerproject/lua-compat-5.2/archive/refs/tags/v0.3.tar.gz) `[627e842c488c1afece4383da9f9a839a7ddbc2f7fedfc456f76e3a590d4ca67f]`
  * [luafilesystem-1_7_0_2.tar.gz](https://github.com/lunarmodules/luafilesystem/archive/refs/tags/v1_7_0_2.tar.gz) `[23b4883aeb4fb90b2d0f338659f33a631f9df7a7e67c54115775a77d4ac3cc59]`
  * [luaposix_34.0.1.tar.gz](https://github.com/luaposix/luaposix/archive/refs/tags/v34.0.1.tar.gz) `[43e7fb1fa4a8b905dd256703aa62116937be72d07d3a46b66a8e9b2264921129]`
  * [Lmod-8.7.24.tar.gz](https://github.com/TACC/Lmod/archive/refs/tags/8.7.24.tar.gz) `[8451267652059b6507b652e1b563929ecf9b689ffb20830642085eb6a55bd539]`

  You can download the packages and verify their hashes using:

  ```sh
  curl -O https://www.lua.org/ftp/lua-5.3.4.tar.gz \
    -Lo lua-compat-5.2-0.3.tar.gz https://github.com/keplerproject/lua-compat-5.2/archive/refs/tags/v0.3.tar.gz \
    -Lo luafilesystem-1_7_0_2.tar.gz https://github.com/lunarmodules/luafilesystem/archive/refs/tags/v1_7_0_2.tar.gz \
    -Lo luaposix_34.0.1.tar.gz https://github.com/luaposix/luaposix/archive/refs/tags/v34.0.1.tar.gz \
    -Lo Lmod-8.7.24.tar.gz https://github.com/TACC/Lmod/archive/refs/tags/8.7.24.tar.gz

  cat > checksums <<EOF
  8451267652059b6507b652e1b563929ecf9b689ffb20830642085eb6a55bd539 Lmod-8.7.24.tar.gz
  f681aa518233bc407e23acf0f5887c884f17436f000d453b2491a9f11a52400c lua-5.3.4.tar.gz
  627e842c488c1afece4383da9f9a839a7ddbc2f7fedfc456f76e3a590d4ca67f lua-compat-5.2-0.3.tar.gz
  23b4883aeb4fb90b2d0f338659f33a631f9df7a7e67c54115775a77d4ac3cc59 luafilesystem-1_7_0_2.tar.gz
  43e7fb1fa4a8b905dd256703aa62116937be72d07d3a46b66a8e9b2264921129 luaposix_34.0.1.tar.gz
  EOF

  sha256sum -c checksums
  ```

Create a build directory with the installation packages and make it your working directory.
Now inspect and run the Lmod bootstrap installations script:
```
   /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/lmod/install/install_lmod.sh
```

## Enable the hpc module

The hpc module is automatically loaded at login and does the bulk of the HPC environment initialization.

These are the steps to enable the hpc module in the HPC-system-specific module tree:
```
HPC_SSE_SYSNAME="$(/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/sbin/hpc_sysname.sh)"
mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/modules"
ln -s "/software/sse2/generic/modules/hpc" "/software/sse2/${HPC_SSE_SYSNAME}/modules/hpc"
```
Note: a new version of the module is created automatically each time you install a new version of ssetools, which needs to be activated by changing the 'default' symlink in the hpc module directory.
(See the instructions written out when running that script.)

## SSE initialization scripts

The initialization scripts set up environment variables that configure Lmod, and then they activate Lmod and load the HPC module.
They are intended for initialization that is unlikely to change more than a few times over the lifetime of the system.

Prerequisites for the HPC system base initialization:

- The system base initialization should already set either the env variable `HPC_RESOURCE_NAME` or `NSC_RESOURCE_NAME` to a "name" for the system (e.g., tetralith)

As a precaution against accidentally changing the SSE initialization scripts on a system, they must be copied into place (rather than run or symlinked directly in the ssetools repo).
Follow this recipe to copy the bash and tcsh init scripts and set up symlinks:
```
  mkdir -p /software/sse2/init/sse_init_files/.
  cp /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/init/hpc_init_sse.sh /software/sse2/init/sse_init_files/hpc_init_sse.sh
  cp /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/init/hpc_init_sse.csh /software/sse2/init/sse_init_files/hpc_init_sse.csh
  ln -s sse_init_files/hpc_sse_init.sh /software/sse2/init/hpc_init_sse.sh
  ln -s sse_init_files/hpc_sse_init.csh /software/sse2/init/hpc_init_sse.csh
```
At this point you can customize these scripts. For example, if you decided not to initialize the `HPC_RESOURCE_NAME` in the system initialization script, you should add the following to `hpc_init_sse.sh`:
```
export HPC_RESOURCE_NAME="<name>"
```
And the following to `hpc_init_sse.sh`
```
setenv HPC_RESOURCE_NAME "<name>"
```

Also, copy the init helper scripts and Lmod local functions:
```
  mkdir -p /software/sse2/init/scripts/lmod/8.7
  cp "/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/sbin/hpc_sysname.sh" /software/sse2/init/scripts/.
  cp "/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/sbin/hpc_sysid.sh" /software/sse2/init/scripts/.
  cp "/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/lmod/site/SitePackage.lua" /software/sse2/init/scripts/lmod/8.7/.
```

Finally, configure the HPC system profile initialization to execute the respective init script via the symlinks `/software/sse2/init/hpc_init_sse.(c)sh`.
Examples for how to do this are found in `/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/init/profile.d` (may need to be adapted to the system).

**Now re-login as a user to get an initialized SSE environment.**

## Set up the support to run executables in compatibility mode

Containers used for compatibility are configured in the `/software/sse2/${HPC_SSE_SYSNAME}/containers` directory.
Place symbolic links in this directory to the various containers the system should provide.
For example, a typical setup places a symlink from `/software/sse2/${HPC_SSE_SYSNAME}/containers/el7.sif to `/software/admin/containers/images/centos7_compute_node.sif`.
Then a symlink from `/software/admin/containers/images/centos7_compute_node.sif` to `/software/admin/containers/images/centos7-compute_node-11.sif`.

Either at the destination of that symlink or inside `/software/sse2/${HPC_SSE_SYSNAME}/containers`, place a `.meta`-file with information about the container.
(There are advantages both ways: placing the symlink at the final destiation of the symlink allows updating the metadata when containers are updated).

For example, at: `/software/admin/containers/images/centos7-compute_node-11.sif.meta` the following contents could be placed:
```
# el7
description: Mimic the environement of an Enterprise Linux 7 compute node.
bind: /software/sse2:/software/sse:/nobackup:/scratch:/proj
```
(Note that the first header line is required).

The hpc module initialization will read the target of the symlinks in this directory and place the information in the env variable `HPC_CONTAINERS`.
This env variable is used by the ssetools scripts `bin/hpc_container_exec` (and the legacy version `bin/hpc_compat_el7`) which are used to run the executable in containers, e.g., for compatibility mode.

The intent with resolving a symlink into an env variable to be used by this script is to avoid behavior that may be confusing to users if the compatibility image changes during a single user session.
However, in case of, e.g., security issues, the base system initialization on compute nodes MAY set `HPC_CONTAINERS_OVERRIDE`, which, in that case will override `HPC_CONTAINERS` for new jobs that starts after that.
(We only expect this feature to be used in rare exceptional cases.)

## Install dependencies for some of the tools

Some of the tools provided in the `bin` directory needs dependencies installed into the manual directory for this HPC system.
Inspect and run the respective installers:
```
   /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/deps/install/install-pproxy.sh
   /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/deps/install/install-socket_wrapper.sh
```

## Bootstrap Easybuild with toolchains

Easybuild is an integral part for building and providing some of our software modules.
Easybuild needs to be installed, and an intel and gcc build environment set up.

Clone our EasyBuild repos:
```
HPC_SSE_SYSNAME="$(/software/sse2/init/scripts/hpc_sysname.sh)"
mkdir -p "/software/sse2/generic/manual/easybuild/dev"
mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure/"
mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/manual/easybuild/"
(cd "/software/sse2/generic/manual/easybuild/dev"
git clone --branch "master" "datil:/srv/git/sse/nsc-vsc-base.git" "vsc-base"
git -C vsc-base remote add upstream "https://github.com/hpcugent/vsc-base"
git clone --branch "master" "datil:/srv/git/sse/nsc-vsc-install.git" "vsc-install"
git -C vsc-install remote add upstream "https://github.com/hpcugent/vsc-install"
git clone --branch "master" "datil:/srv/git/sse/nsc-easybuild-framework.git" "easybuild-framework"
git -C easybuild-framework remote add upstream "https://github.com/easybuilders/easybuild-framework.git"
git clone --branch "master" "datil:/srv/git/sse/nsc-easybuild-easyblocks.git" "easybuild-easyblocks"
git -C easybuild-easyblocks remote add upstream "https://github.com/easybuilders/easybuild-easyblocks.git"
git clone --branch "master" "datil:/srv/git/sse/nsc-easybuild-easyconfigs.git" "easybuild-easyconfigs"
git -C easybuild-easyconfigs remote add upstream "https://github.com/easybuilders/easybuild-easyconfigs.git"
)
```

Symlink a couple of helper scripts from the ssetools repo to update our dev repos and to install new versions of easybuild:
```
ln -s /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/sbin/easybuild_update_dev_repos.sh /software/sse2/generic/manual/easybuild/.
ln -s /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/sbin/easybuild_install.sh /software/sse2/generic/manual/easybuild/.
```

Run these scripts to get an installation of EasyBuild, including a module named `buildtool-easybuild/<version>`:
```
cd /software/sse2/generic/manual/easybuild/
./easybuild_update_dev_repos.sh
./easybuild_install.sh
ln -s "/software/sse2/generic/modules/buildtool-easybuild" "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildtool-easybuild"
```

### Bootstrap the EasyBuild intel 2023.1.0 compiler (intel-2023a)

We first need to determine the value of the `LD_LIBRARY_PATH` the Intel compilers need.
The first step is to install them, run the setvars script, and then ask easybuild to just rebuild the module with `INTEL_LD_LIBRARY_PATH` set to this, which integrates with the ssetools linker wrapper to set `LD_LIBRARY_PATH` correctly during linktime:
```
module load buildtool-easybuild
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb intel-compilers-2023.1.0.eb --robot --hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man --accept-eula-for=Intel-oneAPI
INTEL_LD_LIBRARY_PATH="$(env -i HOME="$HOME" bash -l -c 'source /software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure/software/intel-compilers/2023.1.0/setvars.sh > /dev/null; echo $LD_LIBRARY_PATH')"
cp intel-compilers-2023.1.0.eb intel-compilers-2023.1.0-fixed.eb
echo "modextravars = {\"HPC_BUILD_LD_LIBRARY_PATH\": \"$INTEL_LD_LIBRARY_PATH\"}" >> intel-compilers-2023.1.0-fixed.eb
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb ./intel-compilers-2023.1.0-fixed.eb --rebuild --module-only --robot --hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man --accept-eula-for=Intel-oneAPI
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb intel-2023a.eb --robot --hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man --accept-eula-for=Intel-oneAPI
```
Finally, add this build environment as a module:
```
publish-eb-buildenv-intel.sh intel/2023a
```

### Bootstrap the EasyBuild gcc build environment (foss-2022a)

The first step is to build a modified EasyBuild OpenSSL-1.1 for the exact same version (OpenSSL 1.1.1k) as available in EL9:
```
module load buildtool-easybuild
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/easybuild/bootstrap/OpenSSL-1.1.eb --robot --hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man
```

Build a modified OpenMPI that fits the hardware used:
```
cp /software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/easybuild/bootstrap/OpenMPI-4.1.4-GCC-11.3.0.eb .
# ** edit OpenMPI-4.1.4-GCC-11.3.0.eb to adjust it as needed **
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb ./OpenMPI-4.1.4-GCC-11.3.0.eb --robot --hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man
```

Use these dependencies to build foss-2022a: 
```
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb foss-2022a.eb --robot --hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man
```

Finally, add this build environment as a module:
```
publish-eb-buildenv-gcc.sh foss/2022a
```


## Building further packages with EasyBuild

Note that when we build EB toolchains we include the argument `--hide-deps=icc,ifort,impi,imkl,iccifort,iimpi,binutils,M4,zlib,Bison,flex,Autoconf,Automake,libtool,numactl,hwloc,gompi,help2man` to *hide* all modules required to bootstrap them that are built using the system compiler.
However, when building and installing other software, this flag should not be included.
EasyBuild configs occasionally depend on these (and other) system-compiler-built software, hence, this will lead to non-hidden duplicates of some of these modules being created.
The creation of these duplicates is intentional: the hidden ones represent the bootstapping dependences, whereas the non-hidden ones are "user-facing".
While one could argue that it is better to not have any dependencies built with the system compiler, it is easier to accept the use of those that are already in use in the upstream easybuild configs.

We thus build software this way:
```
EASYBUILD_PREFIX="/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure" eb <easybuild config to build>.eb --robot
```

## Bootstrap Anaconda modules

One of the more basic needs for users and for some of our models is the ability to use `conda`, which, while Python-oriented, provides a lightweight container system for all kinds of software.

Use the installation scripts provided by ssetools to install Anaconda and Miniforge:
```
/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/conda/install_anaconda.sh
/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/conda/install_miniforge.sh
