# Build and runtime environment

## Overview

The SSE2 build and runtime environment help users avoid common pitfalls on systems that provide multiple concurrent build and MPI setups.
The core design idea is to:

1. Mandate the use of a link wrapper to make all software builds (both for system-wide installation and those done by end-users) use the "rpath" feature to hard-code the build-time run-time search path into executables.

2. Instruct that non-trivial HPC jobs (e.g., that uses MPI) should be run via the `mpprun` tool.
    This tool uses the rpath information embedded in the executables to adjust the environment as necessary to successfully and optimally launch the executable, e.g., by using the correct MPI launcher with a sensible configuration.

This removes the tricky but critical task for end users to set up the correct environment when running an executable.
(At some HPC centers this problem is instead solved by requiring that the relevant environment is first initialized via loading the apropriate modules, but that is arguably easier to get wrong.)

## The hpc module

The SSE2 Lmod setup auto-loads the `hpc` module for every user.
This module uses the [PATH priority feature of Lmod](https://lmod.readthedocs.io/en/latest/077_ref_counting.html#specifying-priorities-for-path-entries) to keep the directory `/software/sse2/generic/manual/ssetools/${SSE_VERSION}/wrappers` first in the user's `PATH`.
This directory contains symbolic links from executable names used by compilers and build systems (`ld`, `ld.gold`, and `real-ld`) to a wrapper script `hpc_ldwrap.sh`.

## Link wrapper

The link wrapper script adjusts the arguments given to the linker to enforce that the relevant rpath information is embedded in the executable.

TODO: document

* hpc_ldwrap.sh

## mpprun

TODO: document

## Compatibility wrappers and other containers

### User-invoked containers

TODO: document

* hpc_compat_el7
* hpc_container_exec

### Automatically wrapped compatibility modules

TODO: document

* sse_create_compat_module.py

## Build tools

### Easybuild

#### Update available version of Easybuild 

To update the easybuild environment in SSE2, do the following:

1. Use ssh with *ssh-agent forwarding* to a login node with write access to `/software/sse2`.

2. Update all the easybuild subrepositories by running (be prepared to confirm a large number of authentications via ssh-agent):
    ```
    cd /software/sse2/generic/manual/easybuild
    easybuild_update_dev_repos.sh
    ```

3. Check the `git log` in all the `easybuild-*` repositories inside `dev/` to see that they are on the desired release. 

4. Install the new version of EasyBuild
    ```
    easybuild_install.sh
    ```

5. Test the new version of EasyBuild by loading the hidden module (see printout after running the installations script).

6. Unhide the new version of EasyBuild
    ```
    cd /software/sse2/generic/modules/buildtool-easybuild
    ls -la 
    ```
   Identify the `.<version>.lua` file that was just installed and rename it to remove the leading dot.

7. Do further testing by `module purge` and `module load buildtool-easybuild`.

TODO: add further documentation

* easybuild_install.sh
* easybuild_update_dev_repos.sh
* publish-eb-app.sh
* publish-eb-buildenv-gcc.sh
* publish-eb-buildenv-intel.sh

## Compatibility

TODO: document

* sse_create_compat_module.py
