# Modules and software installations

SSE2 uses [Lmod](https://lmod.readthedocs.io) for user-loadable modules to access installations of pre-built software on the HPC system.

## Module system setup

The bootsrapping procedure builds and installs a specific Lmod version into `/software/sse2/${HPC_SSE_SYSNAME}/manual/lmod`.
The environment initalization scripts placed in `/software/sse2/init/sse_init_files/sse_init.sh` and `see_init.csh` initalize this Lmod for the user's environment and sets the appropriate environment variables to configure it.
A number of particular site-local Lmod helper functions are placed in `/software/sse2/generic/manual/lmod/8.7/site/SitePackage.lua`.

The configuration of Lmod sets the `MODULEPATH` to: `/software/sse2/generic/${HPC_SSE_SYSNAME}/modules`.
This is a semi-manually curated set of modules exposed to end users of the HPC system.
As described in the [Lmod documentation](https://lmod.readthedocs.io/en/latest/015_writing_modules.html) the structure of this directory is: one "software module directory" for each software provided, and inside these a set of "module files" with `.lua` extensions, one for each version of the software that is made available.
Symbolic links from this directory (either on the software directory or module file level) are used to include modules stored elsewhere.
For example, the bootstrapping procedure places the non-system-specific `hpc` module in `/software/sse2/generic/modules/hpc` and then makes it available for specific systems via such a symbolic link.

## Software installations

The software installations are meant to go into subdirectories of `/software/sse2/${HPC_SSE_SYSNAME}/<build system>/<software name>`, where `<build system>` identifies the build system used for the installation (e.g., `easybuild`).
(For more information on how to build and install software using build systems see: [build-and-runtime-environment](build-and-runtime-environment.md).
Manually curated software installations are placed under `/software/sse2/${HPC_SSE_SYSNAME}/manual/<software name>`.
As part of manually building and/or installing software, a corresponding module to load this software has to be manually created under `/software/sse2/generic/${HPC_SSE_SYSNAME}/modules`.

In some setups one may want more than a single HPC system to share software installations.
This can achived in two ways:

* There may be one or more secondary systems that are meant to always stay in syncronization with a main system, providing all the same software with the same configuration, this may be done by a system-level symbolic link:
    ```
    ln -s /software/sse2/main_system /software/sse2/secondary_system
    ```
    The secondary system is then set up with some form of mount or periodic syncronization of the `main_system` directory.

    However, in this setup, all system-wide installations of software **MUST** be built on the main system and never on the secondary system.
    (Otherwise software may be installed through the symbolic link into the `main_system` directory, which has a major risk of causing confusion and breakage, e.g., via software installers that hard-code paths through that symbolic link.)

* If two systems `system_a` and `system_b` are meant to share some software, but otherwise differ, then both systems are meant to be bootstapped individually to give unique `/software/sse2/system_a` and `/software/sse2/system_b` directories.
    Symbolic should be made for specific modules from `/software/sse2/system_b/modules` into `/software/sse2/system_a/modules`.
    However, for those modules to work,  other directories under `/software/sse2/system_a` must also be present on `system_b`.
    There is no automatic way to do partial syncronizations from `system_a` into `system_b` to cover a specific software and its dependencies.
    (But for some specific software, the dependencies may be know to a degree where this is possible.)
    The safe option is to either syncronize the complete system-level directory or rebuild software independently on each system.

    *Never copy/move directories built for one system into the system-level directory for another system. (Software may be built with hard-coded paths in ways that may break if this is done.)*

## Module directory and file conventions

Some module file conventions are mandated for SSE2 module files:

1. *Module file naming conventions:*
     For all software that isn't a toolchain, the following convention should be followed: `<software identifier>/<software version>[-<feature tag>]-<hpc buildtag>-<toolchain>[-<module tag>]`.
     Where:

     * `<software identifier>`: A short name, e.g., `VASP`, `MATLAB`, `Wannier`, etc.
         No specific capitalization rules are enforced, but the following recommendation applies: (i) follow any existing legacy practices for this specific software; (ii) use the capitalization used internally by the software, or, if no consistent capitalization is promoted or widely in use; (iii) captialize the first letter.

     * `<software version>`: This must start with a number, as this is highly recommended by Lmod.
         Otherwise is should be identical to how the software itself presents its version number.
	 For example, in `Siesta/5.0.0-b1-hpc1-intel-2023a-eb` the software version segment is `5.0.0-b1`, which is the version identifier of this specific installation of Siesta.

     * `<feature tag>`: One or more (preferably lowercase) identifiers hypenated together (e.g., `tag1-tag2-tag3`) to highlights relevant configuration options or optional features included or excluded in this build or installation.
         For example, in `VASP/5.4.4.16052018-wannier90-nsc1-intel-2018a-eb` the feature tag is `wannier90`, which indicates that this is a Wannier90-enabled version of the VASP software.
	 This may include information to indicate dependencies or inclusion of other versioned software, e.g., in `spark/3.5.1-hadoop-3.3.6-hpc1-bdist` the feature tag `hadoop-3.3.6` indicates that this provides version `3.5.1` of the Apache Spark software. 

     * `<hpc buildtag>`: An identifier to disambiguate multiple installations of the software that are otherwise the same.
         For example, if something is found to be deficient in an installation, this tag is used to provide an updated installation while retaining the old one.
	 For future installations this should be `hpc1`, `hpc2`, ..., etc.
	 However, there are legacy installations that use `nsc1`, `nsc2`, ..., etc.
	 
     * `<toolchain>`: For software built using the toolchain module, e.g., `buildenv-intel/2023a-eb`, this should be set to `intel-2023a-eb`.
         For software provided as a binary distribtion, `<toolchain>` should be set to `bdist`.
	 This should generally never be omitted.
	 
     * `-<module tag>`: A final identifier to append information that does not pertain to the software itself or its version, but rather extraneous information that pertain to how this software is provided or launched.
         For example, in `Siesta/4.0.1-nsc1-intel-2018a-eb-compat-el7` the module tag `comapt-el7` signifies that this is is a module made available that automatically launch the software through a Enterprise Linux 7-compatibility container.

     However, many systems have retained modules with legacy naming rules which do not follow the above conventions.

2. *Not using default versions, but rather recommend versions:*
     Historically we have seen problems with making it easy for users to auto-load default versions of software in, e.g., their init files.
     Our design is intended to guide users to make a consious decision of which version of scientific software they use to adhere to good scientific practices (e.g., not swapping out software versions in the middle of a project).
     In particular, it has led to difficult decisions for the HPC staff about when and how to upgrade the default version when backwards-breaking changes are involved, which easily leads to users using unnecessarily old versions.

     Hence, we generally circumvent the Lmod feature to automatically designate a version as the default one by creating a `recommendation.lua` module file in each software module directory and use the Lmod feature of a symbolic link named `default` that points at this file.
     Loading the `recommendation` version (e.g. `module load VASP/recommendation`) should give an error plus a message that describes which version is recommended.
     Our local `SitePackage.lua` file defines a specific help Lmod function for this.
     For legacy reasons, this function is named `nsc_message_recommend`, but for future installations it is intended to be renamed to `hpc_message_recommend`.
     It is used, e.g., like this:
     ```
     nsc_message_recommend("3.5.8-hpc1")`
     ```
     Which makes the module on load print out the recommendation to use the `3.5.8-hpc1` version of the software.
     Apart from the header (see below), this is typically the only content of a `reccomendation.lua` file.

3. *Software information machine-readable headers:*
     Module files can include machine-readable software info as a header on a limited subset of the YAML format.
     The header is placed in a segment of lines at the top of the Lua module file, each line prefixed by `--` (making it a Lua comment), and with the header enclosed by lines containing only: `-- ---`.
     Generic information about the software is placed in the header of the `recommendation.lua` file, and version-specific information (if relevant) is placed inside each the modules for each version.
     Here is an example from `VASP/recommendation.lua`:
     ```
     -- ---
     -- support: tier1
     -- software_name: VASP
     -- software_desc: Electronic structure calculations using PAW-method DFT
     -- software_desc_full: |-
     --     VASP - Vienna ab initio simulation package, "a computer program for atomic scale materials modelling, e.g.
     --     electronic structure calculations and quantum mechanical molecular dynamics, from first principles."
     --
     --     ### License ###
     --     Note: VASP is a licensed software. Please check further information on the licensing page:
     --     [www.nsc.liu.se/software/software-licensing/vasp/](https://www.nsc.liu.se/software/software-licensing/vasp)
     -- homepage: https://vasp.at/
     -- nsc_homepage: /software/docs/vasp/
     -- classes:
     --   - phys
     --   - chem
     -- ---
     ```
     A tool `hpc-modules-extract-metadata.py` reads all these headers and generates aggregated info in a structured format (json or yaml) to be used in, e.g., for our online web documentation system.

