# Upgrading SSE environment to the latest version

## Outline

1. Locate or clone a repository to install from
2. Check out the latest versioned release.
3. Run the install.sh script to create a hidden new version of the hpc module.
4. Test that the new environment works as intended when loaded manually.
5. Make the new hpc module the default one.
6. Test that the upgraded default environment works.

The rest assumes a system installed according to the info in [bootstrap](bootstrap.md).

Except as explicitly otherwise specified, these upgrade instructions should be carried out logged into a user account belonging to the group that grants write rights to the folders under `/software/sse2` on a login server where it is possible to write to this file system.

## Locate or clone a repository to install from

The upgrade can be done using the common dev clone of the ssetools repository in `/software/sse2/generic/manual/ssetools/dev` or from a personal (possibly temporary) clone.
(However, if the common dev clone was created by a different user, you may first need to adjust the file permissions. 
This may require a root shell for executing, e.g., `chmod -R g+rwX /software/sse2/generic/manual/ssetools/dev`.)

To use the common dev clone, once the file permissions are correct, make it your current working directory and fetch any changes:
```
cd /software/sse2/generic/manual/ssetools/dev
git pull --ff-only --tags
```

*Alternatively*, to work from a clone of the ssetools repo in, e.g., your own home directory, do:
```
cd 
git clone git@gitlab.liu.se:NSC/ssetools.git
cd ssetools
```
To work from such a repo that was cloned previously, make it your current working directory and run:
```
git pull --ff-only --tags
```

## Install the new version

Continue from the previous section with the ssetools repo as your current working directory.

Find out what the latest versioned release is:
```
HPC_SSE_VER=$(git describe --tags --abbrev=0)
echo "${HPC_SSE_VER}"
```
Check that the printed version string matches your expectation for the version you intend to upgrade to, which usually means that it is on a pure semantic versioning form prefixed with a 'v', e.g., `v1.2.3`.

Check out the version to be installed.
```
git checkout "tags/${HPC_SSE_VER}"
```

Check the CHANGELOG.md file to see if anything has changed that requires other changes in the system then covered by these upgrade instructions.
(Note: some configuration changes described in CHANGELOG.md may require root rights to carry out.)
```
less CHANGELOG.md
```

Run the installation script.
```
./install.sh
```

Note: 
    
* If the install script describes the version to install with a random-looking suffix, e.g. `v1.2.3-1-gd66926d`, it means that the repostiory is not currently checked out at a version-tagged commit.
       You should normally only install from version-tagged commits.
    
* If the version has a devel suffix, e.g., `v1.2.3-devel`, it means that the repository is not in a clean state, i.e.,
       files in the repo have been edited without been commited.
       You should normally only install from a clean repository.

When the apropriate version of ssetools has been installed, return the repo to the head of the main branch:
```
git checkout main
```

## Test that the new environment works when loaded manually

Use a login to the HPC system that has read-only access to `/software/sse2`.
Load the new version of the module in a clean environment (copy from the printout from the above run of `install.sh` to get this with `<version>` filled out correctly):
```
module --force purge
module add hpc/.<version>.lua"
```

Now test that the environment works as intended.
There is a script to do this by using modules adapted to some of the NSC clusters:
```
mkdir -p ~/Temp
cd ~/Temp
/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/tests/run_minimal_tetralith.sh
```
Note:

* Depending on the HPC system you want to run this on, you may need to adapt this test script to refer to other modules for the tests.
* Running this script will create a tempary subdirectory under `~/Temp` which you may want to clean up afterwards.
* The tests take around 5-10 minutes to finish.
* The tests run by this script are (both for a single node and across at least two nodes):

  - A test MPI program is built with our most standard toolchain.
  - This MPI program is run in an interactive job.
  - This MPI program is run as a batch job.
  - One of our pre-compiled software programs are run.
  - A legacy MPI program is build using the el7 container.
  - This legacy MPI program is run using `mpprun --compat el7`.
  - An automatically wrapped legacy MPI program is run over a single node and multiple nodes.

## Make the new version the default

Make the Lmod default link point at the right version (make sure to replace `<version>` with the correct version identifier):
```
ln -sfn ".<version>.lua" "/software/sse2/generic/modules/hpc/default"
```

## Test that the upgraded default environment works

Log into the HPC system on a standard login node.

Verify that you have the new version of the `hpc` module loaded.
For example:
```
[rar@tetralith1 ~]$ module list

Currently Loaded Modules:
  1) hpc/.1.9.5 (H,S)

  Where:
   S:  Module is Sticky, requires --force to unload or purge
   H:             Hidden Module
```

Now re-do the tests above:
```
mkdir -p ~/Temp
cd ~/Temp
/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/tests/run_minimal_tetralith.sh
```

## Reverting if the tests fail

If any of the tests fails, or you need to revert to a prior version for some other reason.

On the login with write access to `/software/sse2` do:
```
ln -sfn ".<previous version>.lua" "/software/sse2/generic/modules/hpc/default"
```
Log in on a standard login node and follow the steps in [Test that the upgraded default environment works](#test-that-the-upgraded-default-environment-works), but for the old version.
