#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# This script is meant to be the first script initializing a user's environment
# when logging in on the system.
#
# Don't modify this script. Edit the autoloading hpc Lmod module instead
#
# To ssh into the system and bypass all initalization:
#
#   HPC_DISABLE_MODULES=1 ssh -o SendEnv=HPC_DISABLE_MODULES <user>@<host>
#
# To ssh into the system and run a different initialization: 
#
#   HPC_DISABLE_MODULES=1 ssh -o SendEnv=HPC_DISABLE_MODULES <user>@<host>
#   unset HPC_DISABLE_MODULES
#   setenv HPC_LMOD_INIT_CSH /software/sse2/init/hpc_lmod_init_mini.csh
#   source /etc/profile.d/hpc-lmod.sh
#

setenv HPC_SSE_SYSID "`/software/sse2/init/scripts/hpc_sysid.sh`"
setenv HPC_SSE_SYSNAME "`/software/sse2/init/scripts/hpc_sysname.sh`"

# Only set up Lmod once
if (! "$?LMOD_VERSION") then
    setenv LMOD_PACKAGE_PATH "/software/sse2/init/scripts/lmod/8.7"
    setenv LMOD_CASE_INDEPENDENT_SORTING "yes"
    setenv MODULEPATH "/software/sse2/${HPC_SSE_SYSNAME}/modules"
    setenv LMOD_REDIRECT yes
endif

source "/software/sse2/${HPC_SSE_SYSNAME}/manual/lmod/lmod/lmod/init/tcsh"

# Init default modules according to instructions here: https://lmod.readthedocs.io/en/latest/070_standard_modules.html
if ( ! $?__Init_Default_Modules )  then
   setenv __Init_Default_Modules 1
   module --initial_load -q restore
   module --initial_load -q load hpc
else
   module refresh
endif
