#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# This script is meant to be the first script initializing a user's environment
# when logging in on the system.
#
# Don't modify this script. Edit the autoloading hpc Lmod module instead
#
# To ssh into the system and bypass all initalization:
#
#   HPC_DISABLE_MODULES=1 ssh -o SendEnv=HPC_DISABLE_MODULES <user>@<host>
#
# To ssh into the system and run a different initialization: 
#
#   HPC_DISABLE_MODULES=1 ssh -o SendEnv=HPC_DISABLE_MODULES <user>@<host>
#   unset HPC_DISABLE_MODULES
#   export HPC_LMOD_INIT_SH=/software/sse2/init/hpc_lmod_init.sh
#   . /etc/profile.d/hpc-lmod.sh
#

export HPC_SSE_SYSID="$(/software/sse2/init/scripts/hpc_sysid.sh)"
export HPC_SSE_SYSNAME="$(/software/sse2/init/scripts/hpc_sysname.sh)"

# Only set up Lmod once
if [ -z "$LMOD_VERSION" ]; then
    export LMOD_PACKAGE_PATH="/software/sse2/init/scripts/lmod/8.7"
    export LMOD_CASE_INDEPENDENT_SORTING="yes"
    export MODULEPATH="/software/sse2/${HPC_SSE_SYSNAME}/modules"
    export LMOD_REDIRECT=yes
fi

. "/software/sse2/${HPC_SSE_SYSNAME}/manual/lmod/lmod/lmod/init/$(basename $SHELL)"

# Init default modules according to instructions here: https://lmod.readthedocs.io/en/latest/070_standard_modules.html
if [ -z "$__Init_Default_Modules" ]; then
   export __Init_Default_Modules=1;
   module --initial_load -q --no_redirect restore
   module --initial_load -q --no_redirect load hpc
else
   module refresh
fi
