#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# This is an example of an initalization script to place in /etc/profile.d/nsc-lmod.csh
# that executes the /software/sse2/init/hpc_init_sse.csh when logging into the system.

#
# Hack to delay lmod init with thinlinc until you start a terminal
#
if ("$?XDG_SESSION_TYPE") then
    if (("$XDG_SESSION_TYPE" == "x11") && (! "$?XDG_CURRENT_DESKTOP")) then
        set HPC_DISABLE_MODULES = "true"
    endif
endif

#
# Set HPC_LMOD_INIT_BASE before /etc/profile.d/nsc-lmod.*
# if you want to use other directory than /software/sse/init
#
if (! "$?HPC_LMOD_INIT_BASE") then
    setenv HPC_LMOD_INIT_BASE "/software/sse2/init"
endif

#
# Initialize lmod for users with uid >= 1000
#
if (("$uid" >= "1000") && (! "$?HPC_DISABLE_MODULES") && \
        (-d "$HPC_LMOD_INIT_BASE")) then
    if (! "$?HPC_LMOD_INIT_CSH") then
        if (("$?HPC_LMOD_INIT") && (-r "$?HPC_LMOD_INIT")) then
            setenv HPC_LMOD_INIT_CSH \
                "`echo "$HPC_LMOD_INIT" | /usr/bin/sed s/.sh/.csh/`"
            setenv HPC_LMOD_INIT_SH \
                "`echo "$HPC_LMOD_INIT" | /usr/bin/sed s/.csh/.sh/`"
        else
            if (! "$?HPC_LMOD_INIT_CSH_LINK") then
                set HPC_LMOD_INIT_CSH_LINK = \
                    "$HPC_LMOD_INIT_BASE/hpc_init_sse.csh"
            endif
            if (! "$?HPC_LMOD_INIT_SH_LINK") then
                set HPC_LMOD_INIT_SH_LINK = \
                    "$HPC_LMOD_INIT_BASE/hpc_init_sse.sh"
            endif
            setenv HPC_LMOD_INIT_CSH "`readlink -e \
                "$HPC_LMOD_INIT_CSH_LINK"`"
            setenv HPC_LMOD_INIT_SH "`readlink -e \
                "$HPC_LMOD_INIT_SH_LINK"`"
            # For compatibility
            setenv HPC_LMOD_INIT "$HPC_LMOD_INIT_CSH"
        endif
    endif
    if (-r "$HPC_LMOD_INIT_CSH") then
        source "$HPC_LMOD_INIT_CSH"
    else if ((! "$?SLURM_JOBID") && (! "$?SLURM_JOB_ID")) then
        /bin/sh -c "echo 'Error: Could not initialize lmod module system, please contact NSC.' >&2"
        unsetenv HPC_LMOD_INIT
        unsetenv HPC_LMOD_INIT_SH
        unsetenv HPC_LMOD_INIT_CSH
    endif
endif
