#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# This is an example of an initalization script to place in /etc/profile.d/nsc-lmod.sh
# that executes the /software/sse2/init/hpc_init_sse.sh when logging into the system.

# Hack to delay lmod init with thinlinc until you start a terminal
#
if [ "$XDG_SESSION_TYPE" = "x11" ] && [ -z "$XDG_CURRENT_DESKTOP" ]; then
    HPC_DISABLE_MODULES="true"
fi

#
# Set HPC_LMOD_INIT_BASE before /etc/profile.d/nsc-lmod.*
# if you want to use other directory than /software/sse/init
#
if [ -z "$HPC_LMOD_INIT_BASE" ]; then
    export HPC_LMOD_INIT_BASE="/software/sse2/init"
fi

#
# Initialize lmod for users with uid >= 1000
#
if [ "$UID" -ge "1000" ] && [ -z "$HPC_DISABLE_MODULES" ] && \
        [ -d "$HPC_LMOD_INIT_BASE" ]; then
    if [ -z "$HPC_LMOD_INIT_SH" ]; then
        if [ ! -z "$HPC_LMOD_INIT" ] && [ -r "$HPC_LMOD_INIT" ]; then
            export HPC_LMOD_INIT_SH="$(echo "$HPC_LMOD_INIT" | \
                /usr/bin/sed s/.csh/.sh/)"
            export HPC_LMOD_INIT_CSH="$(echo "$HPC_LMOD_INIT" | \
                /usr/bin/sed s/.sh/.csh/)"
        else
            if [ -z "$HPC_LMOD_INIT_SH_LINK" ]; then
                HPC_LMOD_INIT_SH_LINK="$HPC_LMOD_INIT_BASE/hpc_init_sse.sh"
            fi
            if [ -z "$HPC_LMOD_INIT_CSH_LINK" ]; then
                HPC_LMOD_INIT_CSH_LINK="$HPC_LMOD_INIT_BASE/hpc_init_sse.csh"
            fi
            export HPC_LMOD_INIT_SH="$(readlink -e \
                "$HPC_LMOD_INIT_SH_LINK")"
            export HPC_LMOD_INIT_CSH="$(readlink -e \
                "$HPC_LMOD_INIT_CSH_LINK")"
            # For compatibility
            export HPC_LMOD_INIT="$HPC_LMOD_INIT_SH"
        fi
    fi
    if [ -r "$HPC_LMOD_INIT_SH" ]; then
        . "$HPC_LMOD_INIT_SH"
    elif [ -z "$SLURM_JOBID" ] && [ -z "$SLURM_JOB_ID" ]; then
        echo "Error: Could not initialize lmod module system, please contact NSC." >&2
        unset HPC_LMOD_INIT
        unset HPC_LMOD_INIT_SH
        unset HPC_LMOD_INIT_CSH
    fi
fi
