#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# History
#   rar@nsc: 2023 initial version
#

if [ -z "$HPC_SSE_VER" -o ! -d "/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}" ]; then
    echo "The environment variable HPC_SSE_VER must be set to a subdirectory of /software/sse2/generic/manual/ssetools/" >&2
    exit 1
fi

if ! SYSNAME="$(/software/sse2/generic/manual/ssetools/${HPC_SSE_VER}/sbin/hpc_sysname.sh)"; then
    echo "Failed to call hpc_sysname.sh, aborting" >&2
    exit 1
fi
LMODVER=8.7.24

tar -zxvf lua-5.3.4.tar.gz
cd lua-5.3.4
mkdir -p "/software/sse2/${SYSNAME}/manual/lmod/deps/lua/"
sed -i "s%^INSTALL_TOP= /usr/local%INSTALL_TOP=/software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4%" Makefile
make linux
make install
cd ..

export PATH="/software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/bin/:$PATH"
export LUA_DIR="/software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4"

tar -zxvf luaposix_34.0.1.tar.gz
cd luaposix-34.0.1
export CC="gcc -I /software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/include -L /software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/lib -Wl,-rpath=/software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/lib"
build-aux/luke PREFIX=/software/sse2/${SYSNAME}/manual/lmod/deps/luaposix/34.0.1 all
build-aux/luke PREFIX=/software/sse2/${SYSNAME}/manual/lmod/deps/luaposix/34.0.1 install
unset CC
cd ..

tar -zxvf luafilesystem-1_7_0_2.tar.gz
cd luafilesystem-1_7_0_2
sed -i "s%^PREFIX=/usr/local%PREFIX=/software/sse2/${SYSNAME}/manual/lmod/deps/luafilesystem/1.7.0.2%" config
sed -i 's%^LUA_LIBDIR= \$(PREFIX)/lib/lua/5.1%LUA_LIBDIR= $(PREFIX)/lib/lua/5.3%' config
sed -i 's%^LUA_LIBDIR= \$(PREFIX)/lib/lua/5.1%LUA_LIBDIR= $(PREFIX)/lib/lua/5.3%' config
sed -i "s%^CC= gcc%CC= gcc -I /software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/include -L /software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/lib -Wl,-rpath=/software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/lib%" config
make
make install
cd ..

export LUA_PATH="/software/sse2/${SYSNAME}/manual/lmod/deps/luaposix/34.0.1/share/lua/5.3/?/init.lua;/software/sse2/${SYSNAME}/manual/lmod/deps/luaposix/34.0.1/share/lua/5.3/?.lua;;"
export LUA_CPATH="/software/sse2/${SYSNAME}/manual/lmod/deps/lua/5.3.4/lib/lua/5.3/?.so;/software/sse2/${SYSNAME}/manual/lmod/deps/luaposix/34.0.1/lib/lua/5.3/?.so;/software/sse2/${SYSNAME}/manual/lmod/deps/luafilesystem/1.7.0.2/lib/lua/5.3/?.s
o;./?.so"

tar -zxvf "Lmod-${LMODVER}.tar.gz"
cd "Lmod-${LMODVER}"
./configure --prefix="/software/sse2/${SYSNAME}/manual/lmod" --with-fastTCLInterp=no
make
make install
cd ..
