#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.

import os, logging, subprocess

def handle(executable, arguments, args, hwinfo):

    logging.debug("srun handler invoked on: "+str(executable))

    output = {
        "launcher": "srun",
        "warnings": ["Using srun handler that just uses srun without setting up the environment; no MPI module will be loaded, etc."],
        "viable": False,
        "launch_args": [],
        "native": False,
        "container": "*",
    }

    # Skip the warning if we are in a container (since raw srun is supposed to work in compatibility containers)
    if args.container:
        output["warnings"] = []

    p = subprocess.run(['objdump', "-x", executable], capture_output=True, text=True)

    if p.returncode == 0:
        for line in p.stdout.splitlines():
            if line == "":
                continue
            if line.split()[0] == "RPATH":
                break
        else:
            output["warnings"] = ["******************************************************************************","  MPPRUN HAS BEEN INVOKED ON A BINARY EXECUTABLE FOR WHICH RPATH IS NOT SET,","  THE DYNAMIC LIBRARIES USED WILL DEPEND ON THE CURRENT RUNTIME ENVIRONMENT","  WHICH MAY YIELD INCONSISTENT AND UNEXPECTED RESULTS.","******************************************************************************","  A likely reason for this warning is that the binary program","  executed with mpprun has not been built according to the instructions","  for the hpc system. If this is unintentional, please seek help","  on building software via the relevant web pages, or contact support."] + output["warnings"]
    else:
        # Assume objdump error means this is an executable script
        pass

    output["viable"] = True
    logging.info("mpprun srun handler invoked on: "+str(executable))

    return output
