#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.

"""
This is a template mpprun launch manager.

The purpose of a launch manager is to do adjustments of the
environment, command line arguments, etc., which are shared between all use
of that launcher.

Copy it into a name mpprun_launcher_<launcher namer>.py where:

  <launcher name> (string), the name of the launcher, e.g., "srun".

An mpprun launch manager defines one function 'prepare' taking four arguments:

  - executable (str): full path to the executable (binary or script) being executed by mpprun
  - arguments (list of str): the list of arguments that will be passed to that executable
  - args (argparse object): the parsed arguments passed to mpprun
  - spec (dict): the output from the mpprun handler.

The launch manager must return a spec dictionary on the same format as
for an mpprun handler, with the same meaning for the fields.
(See mpprun_handlertemplate.py).

Note:
- The launch manager is supposed to use the standard python 'logging' module to produce informative output
  with 'logging.debug("...") for things shown only at increased verbosity, and
  'logging.info("...")' for things shown per default. Guideline: always start the handler
  with a single logging.debug("") to describe that the handler is invoked.
"""

import logging

def prepare(executable, arguments, args, spec):

    logging.debug("template launch manager invoked on: "+str(executable))

    return spec
