#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# History:
#   rar@nsc: 2023 initial version

# Get the hardware platform (e.g., x86_64, aarch64)
hw_platform=$(uname -m)

# Get the distribution ID (e.g., ubuntu, fedora)
if [ -f /etc/os-release ]; then
    . /etc/os-release
    distro_id=$ID
    distro_version=$(echo "$VERSION_ID" | awk -F. '{print $1}')
elif [ -f /etc/lsb-release ]; then
    . /etc/lsb-release
    distro_id=$DISTRIB_ID
    distro_version=$(echo "$DISTRIB_RELEASE" | awk -F. '{print $1}')
else
    distro_id="unknown"
    distro_version=""
fi

# Adjust naming
if [ "$distro_id" == "rocky" -o "$distro_id" == "centos" ]; then
    distro_id="el"
fi

# Lowercase the distribution ID
distro_id=$(echo "$distro_id" | tr '[:upper:]' '[:lower:]')

# Get CPU flags that may affect binary compatibility
cpu_hash=$(cat /proc/cpuinfo | awk -F: '/^vendor_id\s|^cpu family\s|^model\s|^model name\s/ {gsub(/[ \t]+$/, "", $1); gsub(/^[ \t]+/, "", $2); print $1 ": " $2} /^processor\s:\s1$/ {exit}' | sort -u | sha1sum | cut -c 1-6)

echo "${distro_id}${distro_version}-${hw_platform}-${cpu_hash}"
