#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# History:
#   rar@nsc: 2023 initial version

if [ -z "$1" ]; then
    echo "Usage: $0 <path to module> [<toolchain used to compile it>]"
    echo "  e.g.:"
    echo "  $0 \"/software/sse2/${HPC_SSE_SYSNAME}/easybuild/pure/modules/all/MPI/GCC/11.3.0/OpenMPI/4.1.4/PyMOL/2.5.0.lua gcc-2022a-eb\""
    echo ""
    echo "Note:"
    echo "  - You can find the full path to a module at the top of 'module show <module>'"
    echo "  - For how to 'name' the toolchain, you should refer to NSC policy documentation,"
    echo "    but in short, it should refer to our naming of these toolchains in the"
    echo "    central module tree, i.e., it should either be intel-<year+letter>-eb,"
    echo "    or gcc-<year+letter>-eb, or bdist."
    echo ""
    exit 1
fi

#TAG="nsc$(basename "$1" ".lua" | sed 's/.*-nsc\([0-9]*\)$/\1/')"
TAG="hpc1"
VER="$(basename "$1" ".lua" | sed 's/-hpc\([0-9]*\)$//')"
DIR="$(dirname "$1")"
APP="$(basename "$DIR")"
TOOLCHAIN="$2"
if [ -n "$TOOLCHAIN" ]; then
    TOOLCHAIN="-$2"
fi

mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/modules/$APP/"
cat "$1" | sed 's/^ *load.*$//' | sed 's/^depends_on(.*$//' > "/software/sse2/${HPC_SSE_SYSNAME}/modules/$APP/$VER-${TAG}${TOOLCHAIN}.lua"

if [ ! -e "/software/sse2/${HPC_SSE_SYSNAME}/modules/$APP/recommendation.lua" ]; then
  cat > "/software/sse2/${HPC_SSE_SYSNAME}/modules/$APP/recommendation.lua" <<EOF
nsc_message_recommend()
EOF
  ln -sf "recommendation.lua" "/software/sse2/${HPC_SSE_SYSNAME}/modules/$APP/default"
fi
