#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# History:
#   rar@nsc: 2023 initial version

if [ -z "$1" ]; then
    echo "Usage: $0 <easybuild dev module name> (e.g. foss/2018a)"
    exit 1
fi

if [ -z "$HPC_SSE_SYSNAME" ]; then
    echo "Environment variable HPC_SSE_SYSNAME must be set."
    exit 1
fi

VER=$(basename "$1")

module purge
module load buildtool-easybuild
module load "$1"

mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-gcc"
cat > "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-gcc/$VER-eb.lua" <<EOF
family("compiler")

help([==[

Description
===========
GNU Compiler Collection (GCC) based compiler toolchain, including OpenMPI for MPI support, OpenBLAS (BLAS and LAPACK support), FFTW and ScaLAPACK.
This build environment was composed using EasyBuild (https://easybuilders.github.io/easybuild/) and will
automatically also load the buildtool-easybuild module.

More information
================
 - Homepage: http://gcc.gnu.org/
]==])

whatis([==[Description: GNU Compiler Collection (GCC) based compiler toolchain, including OpenMPI for MPI support, OpenBLAS (BLAS and LAPACK support), FFTW and ScaLAPACK.]==])
whatis([==[Homepage: Homepage: http://gcc.gnu.org/]==])

conflict("buildenv-gcc")

if mode() == 'load' and not isloaded("foss/$VER") then
LmodMessage('***************************************************')
LmodMessage('You have loaded a buildenv module')
LmodMessage('***************************************************')
LmodMessage('The buildenv-gcc module makes available:')
LmodMessage(' - Compilers: gcc, gfortran, etc.')
LmodMessage(' - MPI library with mpi-wrapped compilers: OpenMPI with mpicc, mpifort, etc.')
LmodMessage(' - Numerical libraries: OpenBLAS, LAPACK, ScaLAPACK, FFTW')
LmodMessage('')
LmodMessage('It also makes a set of dependency library modules available via')
LmodMessage('the regular module command. Just do:')
LmodMessage('  module avail')
LmodMessage('to see what is available.')
LmodMessage('')
LmodMessage('NOTE: You shoud never load build environments inside submitted jobs.')
LmodMessage('(with the single exception of when using supercomputer time to compile code.)')
LmodMessage('')
end

if not isloaded("buildtool-easybuild") or mode() == "unload" then
    load("buildtool-easybuild")
end

if not isloaded("foss/$VER") or mode() == "unload" then
    load("foss/$VER")
end

nsc_set_mpivars("gcc")

EOF

if [ ! -e "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-gcc/recommendation.lua" ]; then
cat > "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-gcc/recommendation.lua" <<EOF
LmodMessage('*************** NO MODULE LOADED *****************')
LmodMessage('***** Please also specify desired version ********')
LmodMessage('')
LmodMessage('')
LmodMessage('This is a notice to inform you that NSC has deprecated the use')
LmodMessage('of default modules for this software. You will now instead need')
LmodMessage('to specify version number/name as well.')
LmodMessage('')
LmodMessage('There is no current version recommendation for this software.')
LmodMessage('')
LmodError('No module loaded.')

EOF
ln -sf "recommendation.lua" "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-gcc/default"
fi

echo "NOTE: you may also need to manually create a libmpi.so.hpcinfo file to specify how to run software with the corresponding MPI."





