#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# History:
#   rar@nsc: 2023 initial version

if [ -z "$1" ]; then
    echo "Usage: $0 <easybuild dev module name> (e.g. intel/2018a)"
    exit 1
fi

if [ -z "$HPC_SSE_SYSNAME" ]; then
    echo "Environment variable HPC_SSE_SYSNAME must be set."
    exit 1
fi

VER=$(basename "$1")

module purge
module load buildtool-easybuild
module load "$1"

mkdir -p "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-intel"
cat > "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-intel/$VER-eb.lua" <<EOF
family("compiler")

help([==[

Description
===========
NSC Intel build environment $VER with Intel C/C++ and Fortran compilers, Intel MPI and Intel MKL.
This build environment uses EasyBuild (https://easybuilders.github.io/easybuild/) and will
automatically also load the buildtool-easybuild module.


More information
================
 - Homepage: http://software.intel.com/en-us/intel-cluster-toolkit-compiler/
]==])

whatis([==[Description: NSC Intel build environment $VER with Intel C/C++ and Fortran compilers, Intel MPI and Intel MKL.]==])
whatis([==[Homepage: http://software.intel.com/en-us/intel-cluster-toolkit-compiler/]==])

conflict("buildenv-intel")

if mode() == 'load' and not isloaded("intel/$VER") then
LmodMessage('***************************************************')
LmodMessage('You have loaded a buildenv module')
LmodMessage('***************************************************')
LmodMessage('The buldenv-intel module makes available:')
LmodMessage(' - Compilers: icc, ifort, etc.')
LmodMessage(' - Mpi library with mpi-wrapped compilers: intel mpi with mpicc, mpifort, etc.')
LmodMessage(' - Numerical libraries: intel MKL')
LmodMessage('')
LmodMessage('It also makes a set of dependency library modules available via')
LmodMessage('the regular module command. Just do:')
LmodMessage('  module avail')
LmodMessage('to see what is available.')
LmodMessage('')
LmodMessage('NOTE: You shoud never load build environments inside submitted jobs.')
LmodMessage('(with the single exception of when using supercomputer time to compile code.)')
LmodMessage('')
end 

if not isloaded("buildtool-easybuild") or mode() == "unload" then
    load("buildtool-easybuild")
end

if not isloaded("intel/$VER") or mode() == "unload" then
    load("intel/$VER")
end

nsc_set_mpivars("intel")

EOF

if [ ! -e "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-intel/recommendation.lua" ]; then
cat > "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-intel/recommendation.lua" <<EOF
LmodMessage('*************** NO MODULE LOADED *****************')
LmodMessage('***** Please also specify desired version ********')
LmodMessage('')
LmodMessage('')
LmodMessage('This is a notice to inform you that NSC has deprecated the use')
LmodMessage('of default modules for this software. You will now instead need')
LmodMessage('to specify version number/name as well.')
LmodMessage('')
LmodMessage('There is no current version recommendation for this software.')
LmodMessage('')
LmodError('No module loaded.')
EOF
ln -sf "recommendation.lua" "/software/sse2/${HPC_SSE_SYSNAME}/modules/buildenv-intel/default"
fi

echo "NOTE: you may also need to manually create a libmpi.so.hpcinfo file to specify how to run software with the corresponding MPI."
