#!/bin/bash
#
# This file is part of ssetools
# Copyright © 2023-2024 The ssetools AUTHORS
#
# Use of this source code is governed by a MIT-style license that can be
# found in the LICENSE file of the ssetools repository.
#
# History:
#   rar@nsc: 2023 initial version

set -e

TEST_SCRIPT_DIR_REL="$(dirname -- "${BASH_SOURCE[0]}")"
TEST_SCRIPT_DIR="$(cd -- "${TEST_SCRIPT_DIR_REL}" && pwd)"

export TESTDIR=$(mktemp -d --tmpdir=.)
cd "$TESTDIR"

echo "Note: to run tests using a specific allocation and reservation,"
echo "set environment variables: TEST_SLURM_ALLOC and TEST_SLURM_RESERV"

echo
echo "======================================="
echo "==== Running buildenv and mpprun tests"
echo "======================================="
echo
"$TEST_SCRIPT_DIR/test_toolchain.sh" buildenv-intel/2023a-eb mpiicc

echo
echo "========================="
echo "==== Running compat tests"
echo "========================"
echo
"$TEST_SCRIPT_DIR/test_compat_executable.sh" el7 buildenv-intel/2018a-eb mpiicc

echo
echo "============================================================"
echo "==== Run prebuilt auto-compatibility-wrapped ABINIT software"
echo "============================================================"
echo
"$TEST_SCRIPT_DIR/test_abinit.sh" ABINIT/9.6.2-nsc1-intel-2018a-compat-el7

echo
echo "======================================"
echo "==== Running pre-built module software"
echo "======================================"
echo
"$TEST_SCRIPT_DIR/test_elk.sh" Elk/9.6.8-hpc1-intel-2023a-eb

echo
echo "================================"
echo "==== Minimal tests all passed =="
echo "================================"
exit 0
