/*
 * This file is part of ssetools
 * Copyright © 2023-2024 The ssetools AUTHORS
 *
 * Use of this source code is governed by a MIT-style license that can be
 * found in the LICENSE file of the ssetools repository.
 *
 * History:
 *  rar@nsc: 2023 initial version
 */

#include <mpi.h>
#include <stdio.h>

int main(int argc, char** argv) {
  int process_Rank, size_Of_Cluster, message_Item;

  MPI_Init(&argc, &argv);
  MPI_Comm_size(MPI_COMM_WORLD, &size_Of_Cluster);
  MPI_Comm_rank(MPI_COMM_WORLD, &process_Rank);

  // Test with MPI_Send and MPI_Recv
  if (process_Rank == 0) {
    message_Item = 42;
    for (int i = 1; i < size_Of_Cluster; i++) {
      MPI_Send(&message_Item, 1, MPI_INT, i, 1, MPI_COMM_WORLD);
    }
    printf("Message Sent: %d\n", message_Item);
  } else {
    MPI_Recv(&message_Item, 1, MPI_INT, 0, 1, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
    printf("Process rank %d received: %d\n", process_Rank, message_Item);
  }
  
  // Synchronize before starting the broadcast test
  MPI_Barrier(MPI_COMM_WORLD);

  // Test with MPI_Bcast
  if (process_Rank == 0) {
    message_Item = 99;
    printf("Broadcasting message: %d\n", message_Item);
  }

  MPI_Bcast(&message_Item, 1, MPI_INT, 0, MPI_COMM_WORLD);
  printf("Process rank %d received broadcasted message: %d\n", process_Rank, message_Item);

  MPI_Finalize();
  return 0;
}
